'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var React = require('react');

/*! *****************************************************************************
Copyright (c) Microsoft Corporation. All rights reserved.
Licensed under the Apache License, Version 2.0 (the "License"); you may not use
this file except in compliance with the License. You may obtain a copy of the
License at http://www.apache.org/licenses/LICENSE-2.0

THIS CODE IS PROVIDED ON AN *AS IS* BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
KIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION ANY IMPLIED
WARRANTIES OR CONDITIONS OF TITLE, FITNESS FOR A PARTICULAR PURPOSE,
MERCHANTABLITY OR NON-INFRINGEMENT.

See the Apache Version 2.0 License for specific language governing permissions
and limitations under the License.
***************************************************************************** */

var __assign = function() {
    __assign = Object.assign || function __assign(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p)) t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};

function __rest(s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {
            if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i]))
                t[p[i]] = s[p[i]];
        }
    return t;
}

function __spreadArrays() {
    for (var s = 0, i = 0, il = arguments.length; i < il; i++) s += arguments[i].length;
    for (var r = Array(s), k = 0, i = 0; i < il; i++)
        for (var a = arguments[i], j = 0, jl = a.length; j < jl; j++, k++)
            r[k] = a[j];
    return r;
}

var convertObjectToString = function (classes) { return Object
    .keys(classes)
    .filter(function (key) { return !!classes[key]; })
    .reduce(function (classString, item) { return (classString
    ? "" + classString + (item ? " " + item : '')
    : "" + item); }, ''); };
var classnames = function () {
    var classes = [];
    for (var _i = 0; _i < arguments.length; _i++) {
        classes[_i] = arguments[_i];
    }
    if (classes[0] && typeof classes[0] === 'string') {
        return classes.join(' ');
    }
    return convertObjectToString(classes[0]);
};

var defaultProps = {
    filter: function () { return true; },
    opened: [],
    displayedName: function (node) { return node.data.name; },
    depthGap: 20,
    verticalLineOffset: 5,
    verticalLineStyles: { stroke: 'black', strokeWidth: 1, strokeDasharray: '1 1' },
    horizontalLineStyles: { stroke: 'black', strokeWidth: 1, strokeDasharray: '1 1' },
    gapMode: 'margin',
};

var getDepthPx = function (depth, depthGap) { return depth * depthGap + "px"; };

function styleInject(css, ref) {
  if ( ref === void 0 ) ref = {};
  var insertAt = ref.insertAt;

  if (!css || typeof document === 'undefined') { return; }

  var head = document.head || document.getElementsByTagName('head')[0];
  var style = document.createElement('style');
  style.type = 'text/css';

  if (insertAt === 'top') {
    if (head.firstChild) {
      head.insertBefore(style, head.firstChild);
    } else {
      head.appendChild(style);
    }
  } else {
    head.appendChild(style);
  }

  if (style.styleSheet) {
    style.styleSheet.cssText = css;
  } else {
    style.appendChild(document.createTextNode(css));
  }
}

var css = ".style__hyper-node-arrow-icon___2JsZ4 {\n  transition: transform 0.1s ease;\n  cursor: pointer;\n  margin-right: 4px; }\n  .style__hyper-node-arrow-icon___2JsZ4.style__opened___18SZM {\n    transform: rotate(90deg); }\n";
var styles = {"hyper-node-arrow-icon":"style__hyper-node-arrow-icon___2JsZ4","opened":"style__opened___18SZM","hyperNodeArrowIcon":"style__hyper-node-arrow-icon___2JsZ4"};
styleInject(css);

var Arrow = function (_a) {
    var _b;
    var onClick = _a.onClick, opened = _a.opened;
    return (React.createElement("svg", { className: classnames((_b = {}, _b[styles.hyperNodeArrowIcon] = true, _b[styles.opened] = !!opened, _b)), xmlns: "http://www.w3.org/2000/svg", width: "10", height: "10", viewBox: "0 0 24 24", onClick: onClick },
        React.createElement("path", { d: "M5 3l3.057-3 11.943 12-11.943 12-3.057-3 9-9z" })));
};

var isFunction = function (func) { return func && {}.toString.call(func) === '[object Function]'; };

var css$1 = ".style__hyper-node___1xbGi {\n  display: flex;\n  align-items: center; }\n";
var styles$1 = {"hyper-node":"style__hyper-node___1xbGi","hyperNode":"style__hyper-node___1xbGi"};
styleInject(css$1);

var DefaultNode = function (_a) {
    var _b = _a.displayedName, displayedName = _b === void 0 ? defaultProps.displayedName : _b, node = _a.node, onSelect = _a.onSelect, onToggle = _a.onToggle;
    return (React.createElement("div", { className: styles$1.hyperNode, onClick: onSelect },
        (node.hasChildren() || node.options.async)
            && !node.isLoading()
            && (React.createElement(Arrow, { onClick: onToggle, opened: node.isOpened() && !!node.hasChildren() })),
        node.isLoading() && (React.createElement(Loader, null)),
        React.createElement("div", null, isFunction(displayedName) ? displayedName(node) : node.data[displayedName])));
};

var css$2 = ".style__horizontal-line___3_N8i {\n  position: absolute;\n  top: 0;\n  display: flex;\n  align-items: center;\n  height: 100%; }\n";
var styles$2 = {"horizontal-line":"style__horizontal-line___3_N8i","horizontalLine":"style__horizontal-line___3_N8i"};
styleInject(css$2);

var HorizontalLine = function (_a) {
    var depth = _a.depth, depthGap = _a.depthGap, _b = _a.verticalLineOffset, verticalLineOffset = _b === void 0 ? defaultProps.verticalLineOffset : _b, _c = _a.horizontalLineStyles, horizontalLineStyles = _c === void 0 ? defaultProps.horizontalLineStyles : _c, gapMode = _a.gapMode;
    return (React.createElement("div", { className: styles$2.horizontalLine, style: {
            width: depthGap - verticalLineOffset + "px",
            left: gapMode === 'padding'
                ? verticalLineOffset - depthGap + depth * depthGap + "px"
                : verticalLineOffset - depthGap + "px",
        } },
        React.createElement("svg", { style: {
                height: horizontalLineStyles.strokeWidth + "px",
                width: '100%',
            } },
            React.createElement("line", { x1: "0%", y1: "0%", x2: "100%", y2: "0%", style: horizontalLineStyles }))));
};

var css$3 = ".style__loading-spinner___3WGvQ {\n  transform: scale(0.3);\n  animation: style__loading-spinner___3WGvQ 1s linear infinite;\n  margin-right: 4px; }\n\n@keyframes style__loading-spinner___3WGvQ {\n  from {\n    transform: rotate(0deg); }\n  to {\n    transform: rotate(360deg); } }\n";
var styles$3 = {"loading-spinner":"style__loading-spinner___3WGvQ","loadingSpinner":"style__loading-spinner___3WGvQ"};
styleInject(css$3);

var Loader = function () { return (React.createElement("svg", { className: styles$3.loadingSpinner, xmlns: "http://www.w3.org/2000/svg", width: "10", height: "10", viewBox: "0 0 48 48" },
    React.createElement("g", { fill: "none" },
        React.createElement("path", { id: "track", fill: "#C6CCD2", d: "M24,48 C10.745166,48 0,37.254834 0,24 C0,10.745166 10.745166,0 24,0\n        C37.254834,0 48,10.745166 48,24 C48,37.254834 37.254834,48 24,48 Z M24,44\n        C35.045695,44 44,35.045695 44,24 C44,12.954305 35.045695,4 24,4\n        C12.954305,4 4,12.954305 4,24 C4,35.045695 12.954305,44 24,44 Z" }),
        React.createElement("path", { id: "section", fill: "#3F4850", d: "M24,0 C37.254834,0 48,10.745166 48,24 L44,24 C44,12.954305 35.045695,4 24,4 L24,0 Z" })))); };

var css$4 = ".style__vertical-line___3DAX3 {\n  position: absolute;\n  top: 0;\n  transition: all 0.2s cubic-bezier(0, 1, 0, 1);\n  animation-fill-mode: backwards; }\n  .style__vertical-line___3DAX3 .style__vertical-svg-line___wptrc {\n    transition: all 0.2s cubic-bezier(0, 1, 0, 1);\n    animation-fill-mode: backwards; }\n";
var styles$4 = {"vertical-line":"style__vertical-line___3DAX3","vertical-svg-line":"style__vertical-svg-line___wptrc","verticalLine":"style__vertical-line___3DAX3","verticalSvgLine":"style__vertical-svg-line___wptrc"};
styleInject(css$4);

var VerticalLine = function (_a) {
    var depth = _a.depth, depthGap = _a.depthGap, count = _a.count, nodeHeight = _a.nodeHeight, _b = _a.verticalLineOffset, verticalLineOffset = _b === void 0 ? defaultProps.verticalLineOffset : _b, _c = _a.verticalLineStyles, verticalLineStyles = _c === void 0 ? defaultProps.verticalLineStyles : _c;
    return (React.createElement("div", { className: styles$4.verticalLine, style: { marginLeft: getDepthPx(depth, depthGap), left: verticalLineOffset + "px" } },
        React.createElement("svg", { style: {
                height: Math.floor(count * nodeHeight - (nodeHeight ? nodeHeight / 2 : 0)) + "px",
                width: verticalLineStyles.strokeWidth + "px",
                position: 'absolute',
                transition: 'all 0.2s cubic-bezier(0, 1, 0, 1)',
            } },
            React.createElement("line", { x1: "0%", y1: "0%", x2: "0%", y2: "100%", className: styles$4.verticalSvgLine, style: verticalLineStyles }))));
};

var css$5 = ".style__hyper-tree-view___3CtKm .style__hyper-tree-node-wrapper___1l1NH {\n  position: relative; }\n  .style__hyper-tree-view___3CtKm .style__hyper-tree-node-wrapper___1l1NH .style__hyper-node-wrapper___348bw {\n    position: relative;\n    transition: all 0.2s ease;\n    opacity: 1;\n    cursor: pointer; }\n    .style__hyper-tree-view___3CtKm .style__hyper-tree-node-wrapper___1l1NH .style__hyper-node-wrapper___348bw:hover {\n      background-color: #b8def5; }\n    .style__hyper-tree-view___3CtKm .style__hyper-tree-node-wrapper___1l1NH .style__hyper-node-wrapper___348bw.style__selected___1A1GB {\n      background-color: #b8def5; }\n  .style__hyper-tree-view___3CtKm .style__hyper-tree-node-wrapper___1l1NH .style__child___2_fIN {\n    position: relative;\n    transition: all 0.2s cubic-bezier(0, 1, 0, 1);\n    transform-origin: top;\n    transform-style: flat;\n    max-height: 9999px;\n    transform: scaleY(1); }\n    .style__hyper-tree-view___3CtKm .style__hyper-tree-node-wrapper___1l1NH .style__child___2_fIN.style__collapsed___3-8dN {\n      transform: scaleY(0);\n      max-height: 0; }\n";
var styles$5 = {"hyper-tree-view":"style__hyper-tree-view___3CtKm","hyper-tree-node-wrapper":"style__hyper-tree-node-wrapper___1l1NH","hyper-node-wrapper":"style__hyper-node-wrapper___348bw","selected":"style__selected___1A1GB","child":"style__child___2_fIN","collapsed":"style__collapsed___3-8dN","hyperTreeView":"style__hyper-tree-view___3CtKm","hyperTreeNodeWrapper":"style__hyper-tree-node-wrapper___1l1NH","hyperNodeWrapper":"style__hyper-node-wrapper___348bw"};
styleInject(css$5);

var HyperTreeNode = React.forwardRef(function (_a, ref) {
    var _b, _c;
    var node = _a.node, depth = _a.depth, _d = _a.depthGap, depthGap = _d === void 0 ? defaultProps.depthGap : _d, _e = _a.displayedName, displayedName = _e === void 0 ? defaultProps.displayedName : _e, staticNodeHeight = _a.staticNodeHeight, _f = _a.verticalLineOffset, verticalLineOffset = _f === void 0 ? defaultProps.verticalLineOffset : _f, horizontalLineStyles = _a.horizontalLineStyles, setOpen = _a.setOpen, setSelected = _a.setSelected, _g = _a.gapMode, gapMode = _g === void 0 ? defaultProps.gapMode : _g, renderNode = _a.renderNode;
    var handleClick = React.useCallback(function (e) {
        e.stopPropagation();
        if (setOpen) {
            setOpen(node);
        }
    }, [node, setOpen]);
    var handleSelected = React.useCallback(function () {
        if (setSelected) {
            setSelected(node, !node.isSelected());
        }
    }, [node, setSelected]);
    var additionStyles = {};
    if (staticNodeHeight) {
        additionStyles.height = staticNodeHeight + "px";
    }
    var offsetProp = gapMode === 'padding' ? 'paddingLeft' : 'marginLeft';
    return (React.createElement("div", { className: classnames((_b = {},
            _b[styles$5.hyperNodeWrapper] = true,
            _b[styles$5.selected] = node.isSelected(),
            _b)), style: __assign((_c = {}, _c[offsetProp] = getDepthPx(depth, depthGap), _c), additionStyles), ref: ref },
        renderNode
            ? renderNode({
                offsetProp: offsetProp,
                displayedName: displayedName,
                node: node,
                onSelect: handleSelected,
                onToggle: handleClick,
                nodeRef: ref,
                depth: depth,
                depthGap: depthGap,
            })
            : (React.createElement(DefaultNode, { displayedName: displayedName, node: node, onSelect: handleSelected, onToggle: handleClick })),
        !node.options.root && (React.createElement(HorizontalLine, { horizontalLineStyles: horizontalLineStyles, verticalLineOffset: verticalLineOffset, gapMode: gapMode, depthGap: depthGap, depth: depth }))));
});
var HyperTreeViewChild = function (_a) {
    var _b;
    var node = _a.node, depth = _a.depth, _c = _a.depthGap, depthGap = _c === void 0 ? defaultProps.depthGap : _c, verticalLineOffset = _a.verticalLineOffset, verticalLineStyles = _a.verticalLineStyles, nodeHeight = _a.nodeHeight, props = __rest(_a, ["node", "depth", "depthGap", "verticalLineOffset", "verticalLineStyles", "nodeHeight"]);
    if (!node.hasChildren()) {
        return null;
    }
    return (React.createElement("div", { className: classnames((_b = {}, _b[styles$5.child] = true, _b[styles$5.collapsed] = !node.options.opened, _b)) },
        React.createElement(HyperTreeView, __assign({ data: node.getChildren(), depth: depth + 1, depthGap: depthGap, verticalLineOffset: verticalLineOffset, verticalLineStyles: verticalLineStyles, nodeHeight: nodeHeight }, props)),
        React.createElement(VerticalLine, { depth: depth, depthGap: depthGap, nodeHeight: nodeHeight, verticalLineOffset: verticalLineOffset, verticalLineStyles: verticalLineStyles, count: node.options.currentChilrenCount })));
};
var HyperTreeView = React.forwardRef(function (_a, ref) {
    var data = _a.data, _b = _a.depth, depth = _b === void 0 ? 0 : _b, props = __rest(_a, ["data", "depth"]);
    return (React.createElement("div", { className: styles$5.hyperTreeView }, data.map(function (currentNode) { return (React.createElement("div", { className: styles$5.hyperTreeNodeWrapper, key: currentNode.id },
        React.createElement(HyperTreeNode, __assign({ node: currentNode, depth: depth, ref: ref }, props)),
        React.createElement(HyperTreeViewChild, __assign({ node: currentNode, depth: depth }, props)))); })));
});
var Tree = function (_a) {
    var staticNodeHeight = _a.staticNodeHeight, props = __rest(_a, ["staticNodeHeight"]);
    var nodeRef = React.useRef(null);
    var _b = React.useState(staticNodeHeight || 0), height = _b[0], setHeight = _b[1];
    React.useEffect(function () {
        if (nodeRef.current && !height && !staticNodeHeight) {
            setHeight(nodeRef.current.clientHeight);
        }
    }, [height, staticNodeHeight]);
    return (React.createElement("div", null,
        React.createElement(HyperTreeView, __assign({ depth: 0, ref: nodeRef, staticNodeHeight: staticNodeHeight, nodeHeight: height }, props))));
};

var getRandom = function () { return Math.random().toString(16).slice(-4); };
var uuid = function () { return getRandom() + "-" + getRandom() + "-" + getRandom() + "-" + getRandom() + getRandom(); };

var TreeNode = /** @class */ (function () {
    function TreeNode(data, options, children) {
        this.data = data;
        this.id = data.id || uuid();
        this.options = options;
        this.children = children;
    }
    TreeNode.prototype.getData = function () {
        return this.data;
    };
    TreeNode.prototype.setData = function (data) {
        this.data = data;
    };
    TreeNode.prototype.getChildren = function () {
        return this.children || [];
    };
    TreeNode.prototype.setNodeChildren = function (children, type, reset) {
        if (!this.hasChildren() || reset) {
            this.children = children;
            return this.children;
        }
        if (type === 'first') {
            this.children = __spreadArrays(children, (this.children || []));
            return this.children;
        }
        this.children = __spreadArrays((this.children || []), children);
        return this.children;
    };
    TreeNode.prototype.setChildren = function (children) {
        this.children = children;
    };
    TreeNode.prototype.hasChildren = function () {
        return this.children && this.children.length !== 0;
    };
    TreeNode.prototype.setParent = function (parent) {
        this.options.parent = parent;
    };
    TreeNode.prototype.setChildrenCount = function (count) {
        this.options.childrenCount = count;
    };
    TreeNode.prototype.getFirstChild = function () {
        return this.getChildren()[0] || null;
    };
    TreeNode.prototype.getLastChild = function () {
        return this.children && this.children.length !== 0 ? this.children[this.children.length - 1] : null;
    };
    TreeNode.prototype.setOpened = function (opened) {
        this.options.opened = opened || false;
    };
    TreeNode.prototype.isOpened = function () {
        return this.options.opened;
    };
    TreeNode.prototype.setLoading = function (loading) {
        this.options.loading = loading || false;
    };
    TreeNode.prototype.isLoading = function () {
        return this.options.loading;
    };
    TreeNode.prototype.setSelected = function (selected) {
        this.options.selected = selected || false;
    };
    TreeNode.prototype.isSelected = function () {
        return this.options.selected;
    };
    return TreeNode;
}());
var TreeView = /** @class */ (function () {
    function TreeView(id, data, options) {
        this.id = id;
        this.data = data;
        this.options = options;
        this.enhancedData = [];
        this.hash = uuid();
        this.ids = [];
        if (options.enhance) {
            this.enhance();
        }
    }
    TreeView.prototype.on = function (callback) {
        callback();
    };
    TreeView.prototype.sort = function (tree) {
        tree.sort(this.options.sort);
        return tree;
    };
    TreeView.prototype.staticEnhance = function (data, node) {
        return this._staticEnhance(data, node);
    };
    TreeView.prototype.enhance = function (data) {
        this.enhancedData = this._enhance(data || this.data, this.options);
        return this.enhancedData;
    };
    TreeView.prototype.enhanceNodes = function () {
        var _this = this;
        this.traverse(function (node) {
            _this._calculateChildrenCount(node);
        }, false);
    };
    TreeView.prototype.flat = function (raw) {
        var tree = raw ? this.data : this.enhancedData;
        this.flatData = this._flat(tree);
        this.ids = this.flatData.map(function (item) { return item.id; });
        return this.flatData;
    };
    TreeView.prototype.unselectAll = function () {
        this.traverse(function (node) {
            node.setSelected(false);
        });
    };
    TreeView.prototype.addChildren = function (parentId, data, insertType) {
        if (insertType === void 0) { insertType = 'first'; }
        this._addChildren(this.data, parentId, data, insertType);
    };
    TreeView.prototype.addSibling = function (nodeId, data, insertType) {
        if (insertType === void 0) { insertType = 'after'; }
        this._addSibling(this.data, nodeId, data, insertType);
    };
    TreeView.prototype.remove = function (nodeId, leaveChildren) {
        this._remove(this.data, null, nodeId, leaveChildren);
    };
    TreeView.prototype.traverseRaw = function (callback, deep) {
        if (deep === void 0) { deep = true; }
        if (deep) {
            this._traverseDeep(this.data, callback);
        }
        else {
            this._traverseBreadth(this.data, callback);
        }
    };
    TreeView.prototype.traverse = function (callback, deep) {
        if (deep === void 0) { deep = true; }
        if (deep) {
            this._traverseDeep(this.enhancedData, callback);
        }
        else {
            this._traverseBreadth(this.enhancedData, callback);
        }
    };
    TreeView.prototype.getNodeById = function (id) {
        return this._getNodeById(id, this.enhancedData);
    };
    TreeView.prototype._addChildren = function (data, parentId, node, insertType) {
        for (var i = 0; i < data.length; i += 1) {
            if (data[i].id === parentId) {
                switch (insertType) {
                    case 'first': {
                        data[i].children = __spreadArrays(node, (data[i].children || []));
                        return;
                    }
                    case 'last': {
                        data[i].children = __spreadArrays((data[i].children || []), node);
                        return;
                    }
                    default: return;
                }
            }
            else if (data[i].children && data[i].children.length !== 0) {
                this._addChildren(data[i].children, parentId, node, insertType);
                return;
            }
            return;
        }
    };
    TreeView.prototype._addSibling = function (data, nodeId, node, insertType) {
        for (var i = 0; i < data.length; i += 1) {
            if (data[i].children && data[i].children.length !== 0) {
                var index = data[i].children.findIndex(function (child) { return child.id === nodeId; });
                if (index > -1) {
                    data[i].children.splice(insertType === 'before' ? index : index + 1, 0, node);
                    return;
                }
                this._addSibling(data[i].children, nodeId, node, insertType);
            }
            return;
        }
    };
    TreeView.prototype._flat = function (tree) {
        var _this = this;
        var result = [];
        tree.forEach(function (child) {
            var children = child.children, childData = __rest(child, ["children"]);
            result.push(childData);
            if (children && children.length !== 0) {
                result.push.apply(result, _this._flat(children));
            }
        });
        return result;
    };
    TreeView.prototype._enhance = function (tree, options, parentNode) {
        var _this = this;
        var result = [];
        this.sort(tree).filter(this.options.filter).forEach(function (child) {
            var filteredChildren = child.children && child.children.length !== 0
                ? child.children.filter(_this.options.filter)
                : [];
            var newChild = new TreeNode(child, {
                opened: !!options.defaultOpened && !child.getChildren,
                selected: false,
                root: !parentNode,
                leaf: !filteredChildren,
                hasChildren: filteredChildren.length !== 0,
                async: !!child.getChildren,
                parent: parentNode,
                childrenCount: 0,
                currentChilrenCount: 0,
                loading: false,
            });
            var children = [];
            if (filteredChildren.length !== 0) {
                children = _this._enhance(_this.sort(filteredChildren), options, newChild);
            }
            newChild.setChildren(children);
            _this._calculateChildrenCount(newChild);
            result.push(newChild);
        });
        return result;
    };
    TreeView.prototype._staticEnhance = function (tree, parentNode) {
        var _this = this;
        var result = [];
        tree.forEach(function (child) {
            var curentChildren = child.children || [];
            var newChild = new TreeNode(child, {
                opened: false,
                selected: false,
                root: !parentNode,
                leaf: curentChildren.length === 0,
                hasChildren: curentChildren.length !== 0,
                async: !!child.getChildren,
                parent: parentNode,
                childrenCount: 0,
                currentChilrenCount: 0,
                loading: false,
            });
            var children = [];
            if (curentChildren.length !== 0) {
                children = _this._staticEnhance(curentChildren, newChild);
            }
            newChild.setChildren(children);
            _this._calculateChildrenCount(newChild);
            result.push(newChild);
        });
        return result;
    };
    TreeView.prototype._calculateChildrenCount = function (child) {
        var childrenCount = child.getChildren().reduce(function (acc, c) {
            if (c.isOpened()) {
                acc += c.options.childrenCount;
                return acc;
            }
            return acc;
        }, 0);
        if (child.isOpened()) {
            child.options.childrenCount = child.getChildren().length + childrenCount;
        }
        else {
            child.options.childrenCount = child.getChildren().length;
        }
        var lastChild = child.getLastChild();
        if (lastChild && lastChild.isOpened()) {
            var globalCount = child.options.childrenCount;
            child.options.currentChilrenCount = globalCount - lastChild.options.childrenCount;
        }
        else {
            child.options.currentChilrenCount = child.options.childrenCount;
        }
    };
    TreeView.prototype._remove = function (data, parentNode, nodeId, leaveChildren) {
        for (var i = 0; i < data.length; i += 1) {
            if (data[i].id === nodeId) {
                var nodeToRemove = data[i];
                data.splice(i, 1);
                if (parentNode && nodeToRemove.children && nodeToRemove.children.length !== 0 && leaveChildren) {
                    parentNode.children = leaveChildren === 'start'
                        ? __spreadArrays(nodeToRemove.children, (parentNode.children || [])) : __spreadArrays((parentNode.children || []), nodeToRemove.children);
                }
                return;
            }
            if (data[i].children && data[i].children.length !== 0) {
                this._remove(data[i].children, data[i], nodeId, leaveChildren);
            }
        }
    };
    TreeView.prototype._getNodeById = function (id, tree) {
        for (var i = 0; i < tree.length; i += 1) {
            if (tree[i].id === id) {
                return tree[i];
            }
            if (tree[i].children && tree[i].children.length !== 0) {
                return this._getNodeById(id, tree[i].children);
            }
        }
        return null;
    };
    TreeView.prototype._traverseDeep = function (tree, callback, depth) {
        var _this = this;
        if (depth === void 0) { depth = 0; }
        tree.forEach(function (node, index) {
            callback(node, index, depth);
            if (node.children && node.children.length !== 0) {
                _this._traverseDeep(node.children, callback, depth + 1);
            }
        });
    };
    TreeView.prototype._traverseBreadth = function (tree, callback, depth) {
        var _this = this;
        if (depth === void 0) { depth = 0; }
        tree.forEach(function (node, index) {
            if (node.children && node.children.length !== 0) {
                _this._traverseDeep(node.children, callback, depth + 1);
            }
            callback(node, index, depth);
        });
    };
    return TreeView;
}());

var TreeHandlers = /** @class */ (function () {
    function TreeHandlers() {
        this.trees = {};
    }
    TreeHandlers.prototype.getIds = function () {
        return Object.keys(this.trees);
    };
    TreeHandlers.prototype.safeUpdate = function (id, tree) {
        this.trees[id] = {
            instance: tree,
            handlers: {},
        };
        return this;
    };
    TreeHandlers.prototype.remove = function (id) {
        var _a = this.trees, _b = id, _ = _a[_b], trees = __rest(_a, [typeof _b === "symbol" ? _b : _b + ""]);
        this.trees = trees;
        return this;
    };
    TreeHandlers.prototype.safeUpdateHandler = function (treeId, handlerName, handler) {
        var _a;
        if (this.trees[treeId]) {
            this.trees[treeId].handlers = __assign(__assign({}, this.trees[treeId].handlers), (_a = {}, _a[handlerName] = handler, _a));
        }
        return this;
    };
    TreeHandlers.prototype.removeHandler = function (treeId, handlerName) {
        if (this.trees[treeId]) {
            var _a = this.trees[treeId].handlers, _b = handlerName, _ = _a[_b], handlers = __rest(_a, [typeof _b === "symbol" ? _b : _b + ""]);
            this.trees[treeId].handlers = handlers;
        }
        return this;
    };
    return TreeHandlers;
}());
var treeHandlers = new TreeHandlers();

var defaultOptions = {
    filter: function () { return true; },
    sort: function () { return 0; },
};
var useForceUpdate = function () {
    var _a = React.useState(Object.create(null)), dispatch = _a[1];
    var memoizedDispatch = React.useCallback(function (callback) {
        if (callback) {
            callback();
        }
        dispatch(Object.create(null));
    }, [dispatch]);
    return memoizedDispatch;
};
var useTreeState = function (_a) {
    var id = _a.id, data = _a.data, filter = _a.filter, sort = _a.sort, defaultOpened = _a.defaultOpened, multipleSelect = _a.multipleSelect;
    var forceUpdate = useForceUpdate();
    var treeView = React.useMemo(function () { return new TreeView(id, Array.isArray(data) ? data : [data], {
        defaultOpened: defaultOpened,
        enhance: true,
        filter: filter || defaultOptions.filter,
        sort: sort || defaultOptions.sort,
    }); }, [data, defaultOpened, filter, id, sort]);
    var setLoading = React.useCallback(function (node, loading) {
        node.setLoading(loading);
        forceUpdate();
    }, [forceUpdate]);
    var setSelected = React.useCallback(function (node, selected) {
        if (!multipleSelect && selected) {
            treeView.unselectAll();
        }
        node.setSelected(selected);
        forceUpdate();
    }, [forceUpdate, treeView, multipleSelect]);
    var setChildren = React.useCallback(function (parent, children, type, reset) {
        parent.setNodeChildren(children, type, reset);
        treeView.enhanceNodes();
        forceUpdate();
    }, [treeView, forceUpdate]);
    var setRawChildren = React.useCallback(function (parent, children, type, reset) {
        setChildren(parent, treeView.staticEnhance(children, parent), type, reset);
    }, [setChildren, treeView]);
    var setOpen = React.useCallback(function (node) {
        if (!node) {
            return;
        }
        if (node.data.getChildren && isFunction(node.data.getChildren) && !node.isOpened()) {
            setLoading(node, true);
            var result = node.data.getChildren({ node: node });
            if (result.then) {
                result.then(function (asyncData) {
                    setLoading(node, false);
                    node.setOpened(true);
                    setRawChildren(node, asyncData, 'last', true);
                });
            }
        }
        else {
            node.setOpened(!node.isOpened());
            treeView.enhanceNodes();
        }
        forceUpdate(function () {
            treeView.enhanceNodes();
        });
    }, [forceUpdate, treeView, setLoading, setRawChildren]);
    treeHandlers
        .safeUpdate(id, treeView)
        .safeUpdateHandler(id, 'rerender', forceUpdate)
        .safeUpdateHandler(id, 'setOpen', setOpen)
        .safeUpdateHandler(id, 'setLoading', setLoading)
        .safeUpdateHandler(id, 'setSelected', setSelected)
        .safeUpdateHandler(id, 'setRawChildren', setRawChildren)
        .safeUpdateHandler(id, 'setChildren', setChildren);
    var handlers = React.useMemo(function () { return ({
        setOpen: setOpen,
        setLoading: setLoading,
        setSelected: setSelected,
        setChildren: setChildren,
        setRawChildren: setRawChildren,
    }); }, [setOpen, setLoading, setSelected, setChildren, setRawChildren]);
    return {
        instance: treeView,
        handlers: handlers,
        required: {
            data: treeView.enhancedData,
        },
    };
};

exports.Arrow = Arrow;
exports.DefaultNode = DefaultNode;
exports.HorizontalLine = HorizontalLine;
exports.Loader = Loader;
exports.Tree = Tree;
exports.VerticalLine = VerticalLine;
exports.default = Tree;
exports.treeHandlers = treeHandlers;
exports.useTreeState = useTreeState;
//# sourceMappingURL=index.js.map
